package com.qs.mp.web.controller.api.admin;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.service.IService;
import com.google.common.html.HtmlEscapers;
import com.qs.mp.admin.domain.Coupon;
import com.qs.mp.admin.domain.Goods;
import com.qs.mp.admin.domain.GoodsCategory;
import com.qs.mp.admin.domain.GoodsSku;
import com.qs.mp.admin.domain.param.CouponParam;
import com.qs.mp.admin.domain.param.GoodsParam;
import com.qs.mp.admin.domain.param.GoodsQueryParam;
import com.qs.mp.admin.domain.vo.GoodsListVO;
import com.qs.mp.admin.domain.vo.GoodsVO;
import com.qs.mp.admin.service.IGoodsCategoryService;
import com.qs.mp.admin.service.IGoodsService;
import com.qs.mp.admin.service.IGoodsSkuService;
import com.qs.mp.channel.domain.Channel;
import com.qs.mp.channel.domain.param.ChannelParam;
import com.qs.mp.channel.domain.vo.ChannelOperDataVO;
import com.qs.mp.channel.domain.vo.ChannelVO;
import com.qs.mp.channel.service.IChannelService;
import com.qs.mp.channel.service.IChannelUserRelService;
import com.qs.mp.common.constant.UserConstants;
import com.qs.mp.common.core.domain.AjaxResult;
import com.qs.mp.common.core.page.TableDataInfo;
import com.qs.mp.common.enums.ChannelRoleEnum;
import com.qs.mp.common.enums.CouponStatusEnum;
import com.qs.mp.common.enums.ErrorCodeEnum;
import com.qs.mp.common.enums.GoodsStatusEnum;
import com.qs.mp.common.utils.html.EscapeUtil;
import com.qs.mp.system.domain.SysUser;
import com.qs.mp.system.service.ISysUserService;
import com.qs.mp.utils.SecurityUtils;
import com.qs.mp.web.controller.common.BaseApiController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import ma.glasnost.orika.MapperFacade;

import java.math.BigDecimal;
import java.net.URI;
import java.net.URLDecoder;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.commons.codec.Decoder;
import org.apache.commons.lang3.StringEscapeUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * @auther liugl
 * @create 2022-03-09 23:45:48
 * @describe 商品管理前端控制器
 */
@Api("渠道管理API")
@RestController
@RequestMapping("/api/v1/mp/admin/goods/*")
@Component
public class GoodsMgrController extends BaseApiController {

	@Autowired
	private IGoodsService goodsService;
	
	@Autowired
	private IGoodsSkuService goodsSkuService;
	
	@Autowired
	private IGoodsCategoryService goodsCategoryService;
	
	@Autowired
	private ISysUserService sysUserService;

	@Autowired
	private MapperFacade mapperFacade;


	/**
	 * 查询商品列表, 支持翻页
	 *
	 * @return
	 */
	@PostMapping("/list")
	public TableDataInfo list(@RequestBody GoodsQueryParam queryParam) {
		startPage();
		LambdaQueryWrapper<Goods> queryWrapper = new LambdaQueryWrapper<Goods>();
		queryWrapper.like(null != queryParam && StringUtils.isNotBlank(queryParam.getTitle()), Goods::getTitle, queryParam.getTitle());
		queryWrapper.eq(null != queryParam && StringUtils.isNotBlank(queryParam.getGoodsId()), Goods::getGoodsId, queryParam.getGoodsId());
		// 成本
		queryWrapper.ge(null != queryParam && null != queryParam.getMinCost(), Goods::getCost, queryParam.getMinCost());
		queryWrapper.le(null != queryParam && null != queryParam.getMaxCost(), Goods::getCost, queryParam.getMaxCost());
		// 价格
		queryWrapper.ge(null != queryParam && null != queryParam.getMinValue(), Goods::getValue, queryParam.getMinValue());
		queryWrapper.le(null != queryParam && null != queryParam.getMaxValue(), Goods::getValue, queryParam.getMaxValue());
		// 状态
		queryWrapper.eq(null != queryParam && null != queryParam.getStatus(), Goods::getStatus, queryParam.getStatus());
		queryWrapper.eq(Goods::getIsDeleted, 0);
		List<Goods> goodsList = goodsService.list(queryWrapper);
		TableDataInfo res = getDataTable(goodsList);
		res.setRows(goodsList);
		return res;
	}
	
	/**
	 * 获取商品详情信息
	 *
	 * @param
	 * @return
	 */
	@PostMapping(value = "/detail")
	public AjaxResult getGoodsDetail(@RequestBody JSONObject jsonObject) {
		String goodsId = jsonObject.getString("goodsId");
	    if (StringUtils.isBlank(goodsId)){
	      return error(ErrorCodeEnum.ERROR_CODE_1001);
	    }
		Goods goods = goodsService.getById(goodsId);
		if(null == goods || StringUtils.isBlank(goods.getGoodsId())) {
			 return error(ErrorCodeEnum.ERROR_CODE_1001);
		}
		GoodsVO goodsVo = new GoodsVO();
		BeanUtils.copyProperties(goods, goodsVo);
		// 查询SKU列表
		List<GoodsSku> skuList = new ArrayList<>();
		LambdaQueryWrapper<GoodsSku> queryWrapper = new LambdaQueryWrapper<>();
		queryWrapper.eq(GoodsSku::getGoodsId, goods.getGoodsId());
		queryWrapper.orderByDesc(GoodsSku::getCreatedTime);
		skuList = goodsSkuService.list(queryWrapper);
		if(null != skuList && skuList.size() > 0) {
			goodsVo.setSkuList(skuList);
		}
		return AjaxResult.success(goodsVo);
	}


	/**
	 * 新增商品信息
	 * @param
	 * @return
	 */
	@ApiOperation(value = "新增商品信息", notes = "后台商品管理新增商品")
	@PostMapping("/create")
	public AjaxResult goodsCreate(@Validated @RequestBody GoodsParam goodsParam) {
		if (StringUtils.isNotBlank(goodsParam.getGoodsId())) {
			return AjaxResult.error("该商品已存在");
		}
		Goods goods = mapperFacade.map(goodsParam, Goods.class);
		// 1、校验名称是否重复（商品表）
		LambdaQueryWrapper<Goods> queryWrapper = new LambdaQueryWrapper<>();
		queryWrapper.eq(Goods::getTitle, goods.getTitle());
		queryWrapper.eq(Goods::getIsDeleted, 0);
		int titleCount = goodsService.count(queryWrapper);
		if(titleCount > 0) {
			return AjaxResult.error("商品名称" + goods.getTitle() + "已存在！");
		}
		String description = URLDecoder.decode(goodsParam.getDescription());
		goods.setDescription(description);
		goods.setStatus(GoodsStatusEnum.PUT_INIT);
		if(null == goods.getOriginPrice()) {
			goods.setOriginPrice(goods.getExchangePrice());
		}
		// 多SKU
		List<GoodsSku> skuList = goodsParam.getSkuList();
		if(goods.getMultiSku() == 1) { // 
			if((null == skuList || skuList.size() == 0)) {
				return AjaxResult.error("商品" + goods.getTitle() + "不满足多SKU条件！");
			}else {
				GoodsSku firstGoodsSku = skuList.get(0);
				goods.setOriginPrice(firstGoodsSku.getOriginPrice());
				goods.setExchangePrice(firstGoodsSku.getExchangePrice());
				goods.setValue(firstGoodsSku.getValue());
				goods.setCost(firstGoodsSku.getCost());
				// 累计库存
				int totalQuantity = skuList.stream().mapToInt(GoodsSku::getQuantity).sum();
				goods.setQuantity(totalQuantity);
			}
		}
		// 3.插入数据
		try {
			goodsService.saveGoods(goods, skuList);
		} catch (Exception e) {
			return AjaxResult.error("商品'" + goods.getTitle() + "'新增失败" + e.getMessage());
		}
		return AjaxResult.success("商品'" + goods.getTitle() + "'新增成功");
	}

	/**
	 * 编辑商品信息
	 * @param
	 * @return
	 */
	@ApiOperation(value = "编辑商品信息", notes = "后台商品管理修改商品信息")
	@PostMapping("/update")
	public AjaxResult goodsUpdate(@Validated @RequestBody GoodsParam goodsParam) {
		if (null == goodsParam || StringUtils.isBlank(goodsParam.getGoodsId())) {
			return error(ErrorCodeEnum.ERROR_CODE_1001);
		}
		Goods oldGoods = goodsService.getById(goodsParam.getGoodsId());
		if(null == oldGoods || null == oldGoods.getGoodsId()) {
			return AjaxResult.error("商品'" + oldGoods.getTitle() + "'编辑失败，商品ID异常");
		}
		Goods goods = mapperFacade.map(goodsParam, Goods.class);
		// 1、校验名称是否重复（商品表）
		if(!goods.getTitle().equals(oldGoods.getTitle())) {
			LambdaQueryWrapper<Goods> queryWrapper = new LambdaQueryWrapper<>();
			queryWrapper.eq(Goods::getTitle, goods.getTitle());
			queryWrapper.eq(Goods::getIsDeleted, 0);
			int titleCount = goodsService.count(queryWrapper);
			if(titleCount > 0) {
				return AjaxResult.error("商品名称" + goods.getTitle() + "已存在！");
			}
		}
		String description = URLDecoder.decode(goodsParam.getDescription());
		goods.setDescription(description);
		// 多SKU
		List<GoodsSku> skuList = goodsParam.getSkuList();
		if(goods.getMultiSku() == 1) {
			if((null == skuList || skuList.size() == 0)) {
				return AjaxResult.error("商品" + goods.getTitle() + "不满足多SKU条件！");
			}else {
				GoodsSku firstGoodsSku = skuList.get(0);
				goods.setOriginPrice(firstGoodsSku.getOriginPrice());
				goods.setExchangePrice(firstGoodsSku.getExchangePrice());
				goods.setValue(firstGoodsSku.getValue());
				goods.setCost(firstGoodsSku.getCost());
				// 累计库存
				int totalQuantity = skuList.stream().mapToInt(GoodsSku::getQuantity).sum();
				goods.setQuantity(totalQuantity);
			}
		}
		// 3.插入数据
		try {
			goodsService.updateGoods(goods, skuList);
		} catch (Exception e) {
			return AjaxResult.error("商品'" + goods.getTitle() + "'新增失败" + e.getMessage());
		}
		return AjaxResult.success("商品'" + goods.getTitle() + "'新增成功");
	}

	
	
	/**
	 * 停用、启用商品
	 * @param
	 * @return
	 */
	@ApiOperation(value = "停用、启用代金券信息", notes = "代金券管理编辑代金券")
	@PostMapping("/status")
	public AjaxResult goodsStatus(@RequestBody Goods goods) {
		String goodsId = (null != goods && StringUtils.isNotBlank(goods.getGoodsId()))?goods.getGoodsId():"";
		GoodsStatusEnum status = (null != goods && null != goods.getStatus())?goods.getStatus():null;
		if (StringUtils.isBlank(goodsId)
				|| null == status) {
			return error(ErrorCodeEnum.ERROR_CODE_1001);
		}
		try {
			goodsService.lambdaUpdate().set(Goods::getStatus, goods.getStatus()).eq(Goods::getGoodsId, goodsId).update();
			// 查询代金券信息
		} catch (Exception e) {
			return AjaxResult.error("操作失败");
		}
		return AjaxResult.success("操作成功");
	}
	
	
	/**
	 * 删除商品(假删)
	 *
	 * @param
	 * @return
	 */
	@PostMapping(value = "/remove")
	public AjaxResult removeGoods(@RequestBody JSONObject jsonObject) {
		String goodsId = jsonObject.getString("goodsId");
	    if (StringUtils.isBlank(goodsId)){
	      return error(ErrorCodeEnum.ERROR_CODE_1001);
	    }
		Goods goods = goodsService.getById(goodsId);
		if(null == goods || StringUtils.isBlank(goods.getGoodsId())) {
			 return error(ErrorCodeEnum.ERROR_CODE_1001);
		}
		// 商品未下架， 不允许删除
		if(goods.getStatus().equals("on")) {
			return AjaxResult.error("商品未下架，不允许删除");
		}
		LambdaUpdateWrapper<Goods> updateWrapper = new LambdaUpdateWrapper<>();
		updateWrapper.set(Goods::getIsDeleted, 1);
		updateWrapper.eq(Goods::getGoodsId, goods.getGoodsId());
		boolean res = goodsService.update(updateWrapper);
		if(!res) {
			return AjaxResult.error("商品删除失败");
		}
		return AjaxResult.success("商品删除成功");
	}

}
